﻿using System.Windows;
using System.Windows.Media;
using Steema.TeeChart.WPF;
using Steema.TeeChart.WPF.Drawing;
using Steema.TeeChart.WPF.Styles;

namespace TChartTest
{
    public static class ChartUtils
    {
        private const string ChartFont = "Trebuchet";
        private const double AxesLineThickness = 1.0;
        private const int AxesTitleFontSize = 10;
        private const int AxesLabelFontSize = 8;
        private const double GridLineThickness = 0.1;
        private static readonly Color GridLineColor = Colors.Black;
        private const double SeriesLineThickness = 2.75;

        public static void CreateChartControl(out TChart chart)
        {
            var tchart = new TChart();
            tchart.Header.Visible = false;
            tchart.Loaded += OnLoaded;
            tchart.Unloaded += OnUnloaded;
            tchart.AfterDraw += OnAfterDraw;
            tchart.Width = 375;
            tchart.Height = 250;
            tchart.Margin = new Thickness(0);
            
            chart = tchart;
        }

        private static void OnLoaded(object sender, RoutedEventArgs e)
        {
            var chart = (TChart)sender;
            UpdateChart(chart);
        }

        private static void OnUnloaded(object sender, RoutedEventArgs e)
        {
            var tchart = (TChart)sender;
            tchart.Loaded -= OnLoaded;
            tchart.Unloaded -= OnUnloaded;
            tchart.AfterDraw -= OnAfterDraw;
        }

        private static void OnAfterDraw(object sender, Graphics3D g)
        {

        }

        private static void UpdateChart(TChart chart)
        {
            SetupUI(chart);
            AddSeries(chart);
        }

        private static void SetupUI(TChart chart)
        {
            // clear the chart
            chart.Series.Clear();

            chart.MinWidth = 120;
            chart.MinHeight = 80;
            chart.SetValue(FrameworkElement.WidthProperty, DependencyProperty.UnsetValue);
            chart.SetValue(FrameworkElement.HeightProperty, DependencyProperty.UnsetValue);
            chart.SnapsToDevicePixels = true;
            chart.Aspect.View3D = false;
            chart.Background = new SolidColorBrush(Colors.White);

            // --- set panel settings ---
            chart.Panel.Bevel.Inner = BevelStyles.None;
            chart.Panel.Bevel.Outer = BevelStyles.None;
            chart.Panel.Color = Colors.Transparent;
            chart.Panel.Gradient.Visible = false;
            chart.Walls.Back.Color = Colors.Transparent;
            chart.Walls.Back.Gradient.Visible = false;
            chart.Walls.Left.Color = Colors.Transparent;
            chart.Walls.Left.Gradient.Visible = false;
            chart.Walls.Right.Color = Colors.Transparent;
            chart.Walls.Right.Gradient.Visible = false;

            // --- set header settings ---
            chart.Header.Visible = false;
            chart.Header.Text = string.Empty;
            chart.Header.Font.Color = Colors.Black;
            chart.Header.Font.Name = ChartFont;
            chart.Header.Font.Size = AxesTitleFontSize;
            chart.Header.Font.Bold = true;
            chart.Header.Height = 0;

            // --- set border settings ---
            chart.BorderBrush = new SolidColorBrush(Colors.White);
            chart.BorderThickness = new Thickness(0);

            // --- set axes settings ---
            chart.Axes.Visible = true;

            chart.Axes.Bottom.Title.Font.Name = ChartFont;
            chart.Axes.Bottom.Title.Font.Size = AxesTitleFontSize;
            chart.Axes.Bottom.Labels.Font.Name = ChartFont;
            chart.Axes.Bottom.Labels.Font.Size = AxesLabelFontSize;
            chart.Axes.Bottom.MinorTicks.Visible = false;
            chart.Axes.Bottom.AxisPen.Width = AxesLineThickness;

            chart.Axes.Top.Title.Font.Name = ChartFont;
            chart.Axes.Top.Title.Font.Size = AxesTitleFontSize;
            chart.Axes.Top.Labels.Font.Name = ChartFont;
            chart.Axes.Top.Labels.Font.Size = AxesLabelFontSize;
            chart.Axes.Top.MinorTicks.Visible = false;
            chart.Axes.Top.AxisPen.Width = AxesLineThickness;

            chart.Axes.Left.Title.Font.Name = ChartFont;
            chart.Axes.Left.Title.Font.Size = AxesTitleFontSize;
            chart.Axes.Left.Labels.Font.Name = ChartFont;
            chart.Axes.Left.Labels.Font.Size = AxesLabelFontSize;
            chart.Axes.Left.MinorTicks.Visible = false;
            chart.Axes.Left.AxisPen.Width = AxesLineThickness;

            chart.Axes.Right.Title.Font.Name = ChartFont;
            chart.Axes.Right.Title.Font.Size = AxesTitleFontSize;
            chart.Axes.Right.Labels.Font.Name = ChartFont;
            chart.Axes.Right.Labels.Font.Size = AxesLabelFontSize;
            chart.Axes.Right.MinorTicks.Visible = false;
            chart.Axes.Right.AxisPen.Width = AxesLineThickness;

            // --- set gridlines settings ---
            chart.Axes.Bottom.Grid.Visible = true;
            chart.Axes.Bottom.Grid.Style = DashStyles.Solid;
            chart.Axes.Bottom.Grid.Color = GridLineColor;
            chart.Axes.Bottom.Grid.Width = GridLineThickness;
            chart.Axes.Bottom.MinorGrid.Visible = false;

            chart.Axes.Top.Grid.Visible = true;
            chart.Axes.Top.Grid.Style = DashStyles.Solid;
            chart.Axes.Top.Grid.Color = GridLineColor;
            chart.Axes.Top.Grid.Width = GridLineThickness;
            chart.Axes.Top.MinorGrid.Visible = false;

            chart.Axes.Left.Grid.Visible = true;
            chart.Axes.Left.Grid.Style = DashStyles.Solid;
            chart.Axes.Left.Grid.Color = GridLineColor;
            chart.Axes.Left.Grid.Width = GridLineThickness;
            chart.Axes.Left.MinorGrid.Visible = false;

            chart.Axes.Right.Grid.Visible = true;
            chart.Axes.Right.Grid.Style = DashStyles.Solid;
            chart.Axes.Right.Grid.Color = GridLineColor;
            chart.Axes.Right.Grid.Width = GridLineThickness;
            chart.Axes.Right.MinorGrid.Visible = false;

            // --- set legend settings ---
            chart.Legend.Visible = false;
        }

        private static void AddSeries(TChart chart)
        {
            chart.Header.Visible = false;

            chart.Axes.Top.Visible = false;

            chart.Axes.Left.Visible = true;
            chart.Axes.Left.Title.Visible = true;
            chart.Axes.Left.Title.Text = "Y";
            chart.Axes.Left.AutomaticMinimum = true;
            chart.Axes.Left.MinimumOffset = 0;
            chart.Axes.Left.AutomaticMaximum = true;
            chart.Axes.Left.MaximumOffset = 0;
            chart.Axes.Left.Automatic = true;

            chart.Axes.Bottom.Visible = true;
            chart.Axes.Bottom.Title.Visible = true;
            chart.Axes.Bottom.Title.Text = "X";
            chart.Axes.Bottom.AutomaticMinimum = true;
            chart.Axes.Bottom.MinimumOffset = 0;
            chart.Axes.Bottom.AutomaticMaximum = true;
            chart.Axes.Bottom.MaximumOffset = 0;
            chart.Axes.Bottom.Automatic = true;

            var line = new Line();
            line.Pointer.Style = PointerStyles.Nothing;
            line.Color = Colors.Aqua;
            line.Visible = true;
            line.LinePen.Style = DashStyles.Solid;
            line.LinePen.Width = SeriesLineThickness;
            line.LinePen.Color = Colors.Aqua;

            line.Title = "Test";
            for (var i = 0; i < 1000; i++)
            {
                line.Add(i, i);
            }

            chart.Series.Add(line);
        }
    }
}
